// Connection.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RMESSAGE_CONNECTION_H
#define RMESSAGE_CONNECTION_H

#include <List.h>
#include <Locker.h>
#include <Messenger.h>
#include <NetAddress.h>
#include <NetEndpoint.h>
#include <OS.h>

#include "Message.h"

class BNetBuffer;

class LoopConnectionPreventer;
class Message;
class Request;
class RequestHandler;
class RequestHeader;

class Connection {
public:
								Connection(const BNetAddress& address);
	virtual						~Connection();

			status_t			Init(BNetEndpoint* endPoint = NULL);

			bool				IsValid() const;

			const BNetAddress&	GetRemoteAddress() const;

			void				SetRequestHandler(
									RequestHandler* requestHandler);

			status_t			SendRequest(const Request* request);
			status_t			SendRequest(BMessage* request,
									Message* parameter = NULL);
	
private:
			status_t			Receive(BNetBuffer& buffer, size_t size);
			status_t			Receive(uint8* buffer, size_t size);

			status_t			Send(BNetBuffer& buffer);
			status_t			Send(const uint8* buffer, size_t size);

			status_t			ReceiveRequestHeader(BNetBuffer& buffer,
									RequestHeader& header);
			status_t			_ReceiveMessage(int32 size,
									BMessage** _message);
			status_t			ReceiveRequest(Request& request);

			status_t			_AddMessage(BNetBuffer& buffer,
									const BMessage* message);
			status_t			_RemoveMessage(BNetBuffer& buffer,
									BMessage* message, size_t size);

	static	int32				_ReceiverEntry(void* data);
			int32				_Receiver();

private:
			BNetAddress			fRemoteAddress;
			BNetEndpoint*		fNetEndPoint;
			RequestHandler*		fRequestHandler;
			BLocker				fSendLock;
			thread_id			fReceiver;
			bool				fTerminating;
			bool				fValid;
			bool				fReceivingConnection;
};

#endif	// RMESSAGE_CONNECTION_H
