// ConnectionFactory.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <new>

#include <NetAddress.h>

#include "Connection.h"
#include "ConnectionFactory.h"
#include "HostAddress.h"
#include "RMessageServerDefs.h"

// constructor
ConnectionFactory::ConnectionFactory(int port)
	: fPort(port)
{
}

// destructor
ConnectionFactory::~ConnectionFactory()
{
}

// CreateConnection
status_t
ConnectionFactory::CreateConnection(const HostAddress& _address,
	Connection** _connection)
{
	// get address
	BNetAddress address;
	status_t error = _address.GetNetAddress(address, fPort);
	if (error != B_OK)
		return error;
	// create connection object
	Connection* connection = new(nothrow) Connection(address);
	if (!connection)
		return B_NO_MEMORY;
	// open connection
	error = connection->Init();
	if (error == B_OK)
		*_connection = connection;
	else
		delete connection;
	return error;
}

// CreateConnection
status_t
ConnectionFactory::CreateConnection(BNetEndpoint* endPoint,
	Connection** _connection)
{
	if (!endPoint)
		return B_BAD_VALUE;
	// create connection object
	Connection* connection = new(nothrow) Connection(endPoint->RemoteAddr());
	if (!connection)
		return B_NO_MEMORY;
	// open connection
	status_t error = connection->Init(endPoint);
	if (error == B_OK)
		*_connection = connection;
	else
		delete connection;
	return error;
}

