// HostAddress.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include "HostAddress.h"

// constructor
HostAddress::HostAddress()
	: fNetAddress(),
	  fHashCode(0)
{
}

// constructor
HostAddress::HostAddress(const BNetAddress& address)
	: fNetAddress(address),
	  fHashCode(0)
{
	_ComputeHash();
}

// copy constructor
HostAddress::HostAddress(const HostAddress& other)
	: fNetAddress(other.fNetAddress),
	  fHashCode(other.fHashCode)
{
}

// SetTo
status_t
HostAddress::SetTo(const BNetAddress& address)
{
	fNetAddress = address;
	_ComputeHash();
	return B_OK;
}

// InitCheck
status_t
HostAddress::InitCheck() const
{
	return const_cast<BNetAddress&>(fNetAddress).InitCheck();
}

// GetNetAddress
const BNetAddress&
HostAddress::GetNetAddress() const
{
	return fNetAddress;
}

// GetNetAddress
status_t
HostAddress::GetNetAddress(BNetAddress& netAddress, int port) const
{
	in_addr addr;
	status_t error = fNetAddress.GetAddr(addr);
	if (error != B_OK)
		return error;
	return netAddress.SetTo(addr, port);
}

// GetHashCode
uint32
HostAddress::GetHashCode() const
{
	return fHashCode;
}

// ToString
BString
HostAddress::ToString() const
{
	uint32 ip = B_BENDIAN_TO_HOST_INT32((int32)_GetIP());
	BString str;
	return str << (ip >> 24) << '.' << ((ip >> 16) & 0xff)
		<< '.' << ((ip >> 8) & 0xff) << '.' << (ip & 0xff);
}

// =
HostAddress&
HostAddress::operator=(const HostAddress& other)
{
	fNetAddress = other.fNetAddress;
	fHashCode = other.fHashCode;
	return *this;
}

// ==
bool
HostAddress::operator==(const HostAddress& other) const
{
	return (_GetIP() == other._GetIP());
}

// !=
bool
HostAddress::operator!=(const HostAddress& other) const
{
	return !(*this == other);
}

// _GetIP
uint32
HostAddress::_GetIP() const
{
	in_addr addr;
	if (fNetAddress.GetAddr(addr) != B_OK)
		return 0;
	return addr.s_addr;
}

// _ComputeHash
void
HostAddress::_ComputeHash()
{
	fHashCode = _GetIP();
}

