// HostAddress.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RMESSAGE_HOST_ADDRESS_H
#define RMESSAGE_HOST_ADDRESS_H

#include <NetAddress.h>
#include <String.h>

class HostAddress {
public:
								HostAddress();
								HostAddress(const BNetAddress& address);
								HostAddress(const HostAddress& other);

			status_t			SetTo(const BNetAddress& address);
			status_t			InitCheck() const;

			const BNetAddress&	GetNetAddress() const;
			status_t			GetNetAddress(BNetAddress& netAddress,
									int port) const;

			uint32				GetHashCode() const;

			BString				ToString() const;

			HostAddress&		operator=(const HostAddress& other);

			bool				operator==(const HostAddress& other) const;
			bool				operator!=(const HostAddress& other) const;

private:
			uint32				_GetIP() const;
			void				_ComputeHash();

private:
			BNetAddress			fNetAddress;
			uint32				fHashCode;
};

#endif	// RMESSAGE_HOST_ADDRESS_H
