// LocalMessageDeliverer.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RMESSAGE_LOCAL_MESSAGE_DELIVERER_H
#define RMESSAGE_LOCAL_MESSAGE_DELIVERER_H

#include <Messenger.h>

#include "BlockingQueue.h"

class LocalMessageRequest {
public:
								LocalMessageRequest();
								LocalMessageRequest(BMessenger target,
									BMessage* message, BMessage* replyMessage);
								LocalMessageRequest(BMessage* replyToMessage,
									BMessage* message, BMessage* replyMessage);
	virtual						~LocalMessageRequest();

			BMessenger			GetTarget();
			BMessage*			GetReplyToMessage();
			BMessage*			GetMessage();
			BMessage*			GetReplyMessage();

	virtual	void				HandleReply(BMessage* reply);
	virtual	void				DeliveryFailed(status_t error);

protected:
			BMessenger			fTarget;
			BMessage*			fReplyToMessage;
			BMessage*			fMessage;
			BMessage*			fReplyMessage;
};

class LocalMessageDeliverer {
public:
								LocalMessageDeliverer();
								~LocalMessageDeliverer();

			status_t			Init();
			void				Uninit();

			status_t			PushRequest(LocalMessageRequest* request);

			void				Dump();

private:
	static	int32				_DeliveryThreadEntry(void* data);
			int32				_DeliveryThread();

private:
			class DeliveryThreadInfo;

			typedef BlockingQueue<LocalMessageRequest> RequestQueue;

			DeliveryThreadInfo*	fDeliveryThreads;
			RequestQueue		fPendingRequests;
	volatile	bool				fTerminating;
};

#endif	// RMESSAGE_LOCAL_MESSAGE_DELIVERER_H
