// Message.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <Message.h>

#include "Message.h"

// constructor
Message::Message()
	: fMessage(NULL),
	  fData(),
	  fDataMessage(false)
{
}

// constructor
Message::Message(BMessage* message)
	: fMessage(NULL),
	  fData(),
	  fDataMessage(false)
{
	SetTo(message);
}

// constructor
Message::Message(BMessage* message, RemoteData* data)
	: fMessage(NULL),
	  fData(),
	  fDataMessage(false)
{
	SetTo(message, data);
}

// destructor
Message::~Message()
{
	Unset();
}

// SetTo
void
Message::SetTo(BMessage* message)
{
	Unset();
	fMessage = message;
	fDataMessage = false;
}

// SetTo
void
Message::SetTo(BMessage* message, RemoteData* data)
{
	Unset();
	fMessage = message;
	fData.SetTo(data);
	fDataMessage = true;
}

// Unset
void
Message::Unset()
{
	delete fMessage;
	fMessage = NULL;
	fData.Unset();
	fDataMessage = false;
}

// IsDataMessage
bool
Message::IsDataMessage() const
{
	return fDataMessage;
}

// IsEmpty
bool
Message::IsEmpty() const
{
	return (!fMessage && !fDataMessage);
}

// GetMessage
BMessage*
Message::DetachMessage()
{
	BMessage* result = fMessage;
	fMessage = NULL;
	return result;
}

// GetMessage
BMessage*
Message::GetMessage() const
{
	return fMessage;
}

// GetRemoteData
RemoteData*
Message::GetRemoteData() const
{
	return (fData.Get());
}

// GetData
uint8*
Message::GetData() const
{
	return (fData.Get() ? fData->GetData() : NULL);
}

// GetDataSize
size_t
Message::GetDataSize() const
{
	return (fData.Get() ? fData->GetSize() : 0);
}

