// RMessageServer.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RMESSAGE_SERVER_H
#define RMESSAGE_SERVER_H

#include <Application.h>

#include "BlockingQueue.h"
#include "HashMap.h"
#include "HostAddress.h"

class RMessageServer : public BApplication {
public:
								RMessageServer(int port);
	virtual						~RMessageServer();

			status_t			Init();

	virtual	void				ReadyToRun();
	virtual	void				MessageReceived(BMessage* message);
	virtual	status_t			GetSupportedSuites(BMessage* message);
	virtual	BHandler*			ResolveSpecifier(BMessage* message, int32 index,
									BMessage* specifier, int32 form,
									const char* property);

private:
	class ScriptingRequest;

			status_t			_GetRemoteServer(const BNetAddress& _address,
									Connection* receivingConnection,
									RemoteServer** remoteServer);

			void				_HandleConnectRequest(BMessage* request);
			void				_GetSupportedHostSuites(BMessage* request);

			void				_Dump();

	static	int32				_RequestProcessorEntry(void* data);
			int32				_RequestProcessor();

	static	int32				_ScriptingProcessorEntry(void* data);
			int32				_ScriptingProcessor();

	static	int32				_ConnectionListenerEntry(void* data);
			int32				_ConnectionListener();

private:
			typedef BlockingQueue<BMessage> RequestQueue;
			typedef SynchronizedHashMap<HostAddress, RemoteServer>
				RemoteServerMap;

			int					fPort;
			ConnectionFactory	fConnectionFactory;
			LocalMessageDeliverer fMessageDeliverer;
			EventQueue			fEventQueue;
			RemoteServerMap		fRemoteServers;
			RequestQueue		fPendingRequests;
			RequestQueue		fPendingScriptingRequests;
			BNetEndpoint		fIncomingConnections;
			thread_id			fRequestProcessor;
			thread_id			fScriptingProcessor;
			thread_id			fConnectionListener;
			bool				fTerminating;
};

#endif	// RMESSAGE_SERVER_H
