// RemoteData.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <new>

#include <OS.h>

#include "AutoDeleter.h"
#include "Debug.h"
#include "RemoteData.h"

// constructor
RemoteData::RemoteData()
	: fData(NULL),
	  fSize(0)
{
}

// constructor
RemoteData::RemoteData(uint8* data, int32 size)
	: fData(data),
	  fSize(size)
{
}

// destructor
RemoteData::~RemoteData()
{
	Unset();
}

// Create
status_t
RemoteData::Create(int32 size, RemoteData** _data)
{
	if (size < 0 || !_data)
		return B_BAD_VALUE;
	// allocate remote data object
	RemoteData* remoteData = new(nothrow) RemoteData;
	if (!remoteData)
		return B_NO_MEMORY;
	ObjectDeleter<RemoteData> deleter(remoteData);
	// allocate an area large enough
	uint8* data = NULL;
	if (size > 0) {
		int32 areaSize = (size + B_PAGE_SIZE - 1) / B_PAGE_SIZE * B_PAGE_SIZE;
		area_id area = create_area("rmessage data", (void**)&data,
			B_ANY_ADDRESS, areaSize, B_NO_LOCK, B_READ_AREA | B_WRITE_AREA);
		if (area < 0)
			RETURN_ERROR(area);
	}
	// set result
	remoteData->SetTo(data, size);
	deleter.Detach();
	*_data = remoteData;
	return B_OK;
}

// SetTo
void
RemoteData::SetTo(uint8* data, int32 size)
{
	Unset();
	fData = data;
	fSize = size;
}

// Unset
void
RemoteData::Unset()
{
	if (fData && fSize > 0) {
		area_id area = area_for(fData);
		if (area >= 0)
			delete_area(area);
	}
	fData = NULL;
	fSize = 0;
}

