// Request.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RMESSAGE_REQUEST_H
#define RMESSAGE_REQUEST_H

#include "Message.h"
#include "Reference.h"

// RequestHeader
struct RequestHeader {
								RequestHeader();

			int32				GetOverallSize() const;

			status_t			Write(BNetBuffer& buffer) const;
			status_t			Read(BNetBuffer& buffer);

	static	int32				GetHeaderSize();

			int32				requestSize;
			uint32				parameterType;
			int32				messageSize;
			int32				dataSize;
			int32				id;			// debugging only
};

// Request
class Request : public Referencable {
public:
								Request();
	explicit					Request(BMessage* request,
									Message* parameter = NULL);
								~Request();

			void				GetHeader(RequestHeader* header) const;

			void				SetRequestMessage(BMessage* request);
			BMessage*			GetRequestMessage() const;

			void				SetParameter(Message* parameter);
			Message*			GetParameter() const;

private:
								Request(const Request& request);

private:
			BMessage*			fRequestMessage;
			MessageReference	fParameter;
};

typedef Reference<Request> RequestReference;

#endif	// RMESSAGE_REQUEST_H
