// remote-roster.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Entry.h>
#include <List.h>
#include <Roster.h>

#include <RemoteRoster.h>

static const char* kUsage = "Usage: remote-roster <remote host>\n";

// main
int
main(int argc, char** argv)
{
	// check parameter
	if (argc != 2) {
		fprintf(stderr, kUsage);
		return 1;
	}
	if (!strcmp(argv[1], "--help") || !strcmp(argv[1], "-h")) {
		printf(kUsage);
		return 0;
	}
	const char* remotePeer = argv[1];
	// get address
	BNetAddress address;
	status_t error = address.SetTo(remotePeer);
	if (error != B_OK) {
		fprintf(stderr, "Failed to get address of peer `%s': %s\n",
			remotePeer, strerror(error));
		return 1;
	}
	// init remote roster
	RemoteRoster roster;
	error = roster.SetTo(&address);
	if (error != B_OK) {
		fprintf(stderr, "Failed to connect to peer `%s': %s\n",
			remotePeer, strerror(error));
		return 1;
	}
	// get teams
	BList teams;
	error = roster.GetAppList(&teams);
	if (error != B_OK) {
		fprintf(stderr, "Failed to get list of running applications: %s\n",
			strerror(error));
		return 1;
	}
	// print heading
	printf("                               name thread team port flags\n");
	printf("-- -------------------------------- ------ ---- ---- -----\n");
	// print apps
	int32 count = teams.CountItems();
	int32 index = 0;
	for (int32 i = 0; i < count; i++) {
		team_id team = (team_id)teams.ItemAt(i);
		app_info info;
		if (roster.GetRunningAppInfo(team, &info) == B_OK) {
			printf("%2ld %32s %6ld %4ld %4ld %5lx (%s)\n", index, info.ref.name,
				info.thread, info.team, info.port, info.flags, info.signature);
	 		index++;
	 	}
	}
	return 0;
}
