// InputServerConnection.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <string.h>

#include "Debug.h"
#include "InputServerConnection.h"

// constructor
InputServerConnection::InputServerConnection()
	: BHandler(),
	  fTarget()
{
}

// destructor
InputServerConnection::~InputServerConnection()
{
}

// Init
status_t
InputServerConnection::Init(BMessenger inputServer, const char* looperName)
{
	// prepare a scripting message
	BMessage scriptingMessage(B_GET_PROPERTY);
	status_t error = scriptingMessage.AddSpecifier("Messenger");
	if (error != B_OK) {
		FATAL(("Failed to add specifier: %s\n", strerror(error)));
		return error;
	}
	error = scriptingMessage.AddSpecifier("Looper", looperName);
	if (error != B_OK) {
		FATAL(("Failed to add specifier: %s\n", strerror(error)));
		return error;
	}
	// send the scripting message
	BMessage reply;
	error = inputServer.SendMessage(&scriptingMessage, &reply);
	if (error != B_OK) {
		FATAL(("Failed to send scripting message: %s\n", strerror(error)));
		return error;
	}
	// analyze the reply
	if (reply.what != B_REPLY) {
		FATAL(("Bad reply to scripting message\n"));
		return B_ERROR;
	}
	error = reply.FindMessenger("result", &fTarget);
	if (error != B_OK) {
		FATAL(("Failed to get input server add-on looper messenger: %s\n",
			strerror(error)));
		return error;
	}
	return error;
}

