// RDesktopServerConnection.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RDESKTOP_SERVER_CONNECTION_H
#define RDESKTOP_SERVER_CONNECTION_H

#include <Handler.h>
#include <Messenger.h>

class BBitmap;

class RDesktopServerModel;
class RDesktopView;

class RDesktopServerConnection : public BHandler {
public:
								RDesktopServerConnection(
									RDesktopServerModel* model);
								~RDesktopServerConnection();

			status_t			Init(RemoteRoster* roster);

	virtual	void				MessageReceived(BMessage* message);

private:
			void				_HandleConfigUpdate(BMessage* message);
			void				_HandleGraphicsUpdate(BMessage* message);
			void				_DesktopConfigChanged(BRect screenFrame,
									color_space colorSpace);
			void				_UpdateScreen(BRect updateFrame,
									color_space colorSpace, const void* data,
									size_t size, int32 bpr, int32 dataOffset,
									bool delta);

			status_t			_Decompress(const void* data, int32 size,
									int32* decompressedSize);

private:
			RDesktopServerModel*	fModel;
			int32				fToken;
			BMessenger			fRequestMessenger;
			BRect				fScreenFrame;
			color_space			fScreenColorSpace;
			BBitmap*			fScreenShot;
			uint8*				fDecompressionBuffer;
			int32				fDecompressionBufferSize;
};

#endif	// RDESKTOP_SERVER_CONNECTION_H
