// RDesktopServerModel.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RDESKTOP_SERVER_MODEL_H
#define RDESKTOP_SERVER_MODEL_H

#include <Locker.h>
#include <GraphicsDefs.h>
#include <Rect.h>

#include "AutoLocker.h"
#include "HashMap.h"
#include "Messenger.h"

class BBitmap;

enum {
	RDESKTOP_SCREEN_CONFIG		= 0x01,
	RDESKTOP_SCREEN_GRAPHICS	= 0x02,
};

class RDesktopServerModel {
public:
								RDesktopServerModel(bool gfxEnabled,
									bool mouseEnabled, bool keyboardEnabled);
	virtual						~RDesktopServerModel();

			bool				Lock();
			void				Unlock();

			bool				IsGfxEnabled() const;
			bool				IsMouseEnabled() const;
			bool				IsKeyboardEnabled() const;

			status_t			SetScreenConfig(BRect frame,
									color_space colorSpace);
			void				ScreenUpdated();

			BRect				GetScreenFrame() const;
			color_space			GetColorSpace() const;
			BBitmap*			GetScreen() const;

			status_t			AddObserver(BMessenger observer);
			void				RemoveObserver(BMessenger observer);
			uint32				ClearChangeMask(BMessenger observer);

private:
			void				_AddToChangeMasks(uint32 changes);

private:
			class Observer;
			typedef HashMap<Messenger, uint32> ObserverMap;

			BLocker				fLock;
			bool				fGfxEnabled;
			bool				fMouseEnabled;
			bool				fKeyboardEnabled;
			BRect				fScreenFrame;
			color_space			fScreenColorSpace;
			BBitmap*			fScreen;
			ObserverMap			fObservers;
			bool				fModified;
};

// RDesktopServerModelLocker
typedef AutoLocker<RDesktopServerModel,
	AutoLockerStandardLocking<RDesktopServerModel> >
		RDesktopServerModelLocker;

#endif	// RDESKTOP_SERVER_MODEL_H
