// RDesktopView.h
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#ifndef RDESKTOP_VIEW_H
#define RDESKTOP_VIEW_H

#include <Messenger.h>
#include <View.h>

#include "BlockingQueue.h"

class BBitmap;
class BMessageRunner;

class InputMessageHandler;
class RDesktopServerModel;

class RDesktopView : public BView {
public:
								RDesktopView(RDesktopServerModel* model,
									InputMessageHandler* inputHandler,
									BRect frame, uint32 resizeMask,
									uint32 flags);
								~RDesktopView();

	virtual	void				MessageReceived(BMessage* message);

	virtual	void				Draw(BRect updateRect);
	virtual	void				FrameResized(float width, float height);
	virtual	void				KeyDown(const char *bytes, int32 numBytes);
	virtual	void				MouseMoved(BPoint where, uint32 code,
									const BMessage *message);
	virtual	void				MouseDown(BPoint where);
	virtual	void				MouseUp(BPoint where);
	virtual	void				AttachedToWindow();
	virtual	void				DetachedFromWindow();

			void				SetFullScreen(bool fullScreen);

private:
			void				_MouseEvent(uint32 what, BPoint where);

			void				_UpdateScrollers();

			void				_ScreenConfigChanged();
			void				_UpdateScreenViewFrame();

private:
			RDesktopServerModel*	fModel;
			InputMessageHandler*	fInputHandler;
			BRect					fScreenFrame;
			BRect					fScreenViewFrame;
			bool					fIsFullScreen;
};

#endif	// RDESKTOP_VIEW_H
