// RDesktopInputDevice.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <new>

#include <Looper.h>
#include <add-ons/input_server/InputServerDevice.h>

#include "RDesktopServerDefs.h"

class RDesktopInputDevice;

// RDesktopInputDeviceLooper
class RDesktopInputDeviceLooper : public BLooper {
public:
								RDesktopInputDeviceLooper(
									RDesktopInputDevice* device);

	virtual	void				MessageReceived(BMessage* message);

private:
			RDesktopInputDevice*	fDevice;
};

// RDesktopInputDevice
class RDesktopInputDevice : public BInputServerDevice {
public:
								RDesktopInputDevice();
	virtual						~RDesktopInputDevice();

	virtual	status_t			Start(const char *device, void *cookie);
	virtual	status_t			Stop(const char *device, void *cookie);

private:
			RDesktopInputDeviceLooper*	fLooper;

};

// instantiate_input_device
extern "C" _EXPORT
BInputServerDevice*
instantiate_input_device()
{
	return new(nothrow) RDesktopInputDevice;
}

// RDesktopInputDeviceLooper

// constructor
RDesktopInputDeviceLooper::RDesktopInputDeviceLooper(
	RDesktopInputDevice* device)
	: BLooper(kRDesktopInputDeviceLooperName),
	  fDevice(device)
{
}

// MessageReceived
void
RDesktopInputDeviceLooper::MessageReceived(BMessage* message)
{
	switch (message->what) {
		case MSG_INPUT_DEVICE_CONNECT:
		{
			bigtime_t offset = 0;
			bigtime_t when;
			if (message->FindInt64("when", &when) == B_OK)
				offset = real_time_clock() - when;
			BMessage reply(MSG_REPLY);
			reply.AddInt64("time offset", offset);
			message->SendReply(&reply);
			break;
		}
		case B_MOUSE_MOVED:
		case B_MOUSE_DOWN:
		case B_MOUSE_UP:
		case B_KEY_DOWN:
		case B_KEY_UP:
		case B_UNMAPPED_KEY_DOWN:
		case B_UNMAPPED_KEY_UP:
		case B_MODIFIERS_CHANGED:
		{
			bigtime_t when;
			if (message->FindInt64("when", &when) == B_OK) {
				bigtime_t now = real_time_clock();
				if (now < when)
					message->ReplaceInt64("when", now);
			}
			fDevice->EnqueueMessage(new BMessage(*message));
			break;
		}
		default:
			BLooper::MessageReceived(message);
			break;
	}
}


// RDesktopInputDevice

// constructor
RDesktopInputDevice::RDesktopInputDevice()
	: fLooper(NULL)
{
	input_device_ref mouse = { "RDesktopInputDevice", B_POINTING_DEVICE, NULL };
	input_device_ref *devices[] = {
		&mouse, NULL
	};
	RegisterDevices(devices);
}

// destructor
RDesktopInputDevice::~RDesktopInputDevice()
{
}

// Start
status_t
RDesktopInputDevice::Start(const char *device, void *cookie)
{
	fLooper = new(nothrow) RDesktopInputDeviceLooper(this);
	if (fLooper) {
		fLooper->Run();
		return B_OK;
	}
	return B_NO_MEMORY;
}

// Stop
status_t
RDesktopInputDevice::Stop(const char *device, void *cookie)
{
	if (fLooper) {
		fLooper->Lock();
		fLooper->Quit();
		fLooper = NULL;
	}
	return B_OK;
}

