// simpleTest.cpp
// 
// Copyright (c) 2004, Ingo Weinhold (bonefish@cs.tu-berlin.de)
// 
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of a copyright holder shall
// not be used in advertising or otherwise to promote the sale, use or other
// dealings in this Software without prior written authorization of the
// copyright holder.

#include <stdio.h>
#include <string.h>

#include <Application.h>
#include <NetAddress.h>

#include <RemoteRoster.h>

static const char* kUsage = "Usage: %s <remote host>\n";

int
main(int argc, char** argv)
{
	// check parameter
	if (argc != 2) {
		fprintf(stderr, kUsage, argv[0]);
		return 1;
	}
	if (!strcmp(argv[1], "--help") || !strcmp(argv[1], "-h")) {
		printf(kUsage, argv[0]);
		return 0;
	}
	const char* remotePeer = argv[1];
	BApplication app("application/x-vnd.bonefish.rmessage-test1");
	// init remote roster
	printf("initializing roster...\n");
	RemoteRoster roster(remotePeer);
	status_t error = roster.InitCheck();
	if (error != B_OK) {
		printf("Failed to init remote roster: %s\n", strerror(error));
		return 1;
	}
	// get the tracker team
	printf("getting tracker team...\n");
	team_id team = roster.TeamFor("application/x-vnd.Be-TRAK");
	if (team < 0) {
		error = team;
		printf("Failed to get team for Tracker: %s\n", strerror(error));
		return 1;
	}
	printf("Tracker team is: %ld\n", team);
	// get tracker messenger
	printf("getting tracker messenger...\n");
	BMessenger messenger;
	error = roster.GetMessenger(team, &messenger);
	if (error != B_OK) {
		printf("Failed to get messenger for Tracker: %s\n", strerror(error));
		return 1;
	}
	// send quit to Tracker
	printf("sending message...\n");
	error = messenger.SendMessage(B_QUIT_REQUESTED);
	if (error != B_OK) {
		printf("Failed to send message to Tracker: %s\n", strerror(error));
		return 1;
	}
	return 0;
}
